//******************************************************************************
// Projeto - Teste_Ringcomm - comunicação em anel via RS-232
//
// Desenvolvido por: Eng. Márcio José Soares
// versão: 1.0 - 18/05/2023
//
// Plataforma: Placa PE-PIC16F628A by Arne
// Auxiliares: Matriz 8x8 com MAX7219
//
// Compilador: XC8 v 2.40
// IDE       : MPLABX (Linux)
//
// Pinos:
// PORTA:
// RA0 - S1 - envia para STM32
// RA1 - S2 - envia para arduino
// RA2 - S3 - envia para Placa Z80 1990
// RA3 - S4 - sem função
// RA4 - CS    MAX7219
//
// PORTB:
// RB0 - Clock MAX7219
// RB1 - RX
// RB2 - TX
// RB3 - Dados in MAX7219 (MOSI)
// RB4 - LED1
// RB5 - LED2
// RB6 - LED3
// RB7 - Dados out SPI (MISO)

//******************************************************************************
// 
// Últimas alterações:
//  18/05/2023:
//      - criado esse projeto
//  em 23/05/2023:
//		- inicio dos testes com Arduino e PIC
//      - inserido saída por timeout
//		- inserido verificação de erro no nr do dispositivo
//        para evitar mensagem se repetindo
//
//******************************************************************************

//******************************************************************************
// Bits de configuração
//******************************************************************************
#pragma config FOSC = XT        // Seleção do oscilador (Oscilador externo: cristal)
#pragma config WDTE = OFF       // WDT desabilitado
#pragma config PWRTE = OFF       // Power-up Timer habilitado
#pragma config MCLRE = ON       // reset externo via resistor em MCLR 
#pragma config BOREN = OFF      // Brown-out Detect BOD desabilitado
#pragma config LVP = OFF        // Low-Voltage Programming desabilitado - I/O RB4
#pragma config CPD = OFF        // EEPROM desabilitada
#pragma config CP = OFF         // Flash desabilitada

//******************************************************************************
// Arquivos incluídos no módulo
//******************************************************************************
#include "my_defines.h"
#include "pic_SPI.h"
#include "MAX7219.h"
#include "ring_comm.h"

//******************************************************************************
// Constantes globais do módulo
//******************************************************************************

//******************************************************************************
// Constantes externas do módulo
//******************************************************************************

//******************************************************************************
// Variáveis globais do módulo
//******************************************************************************

//******************************************************************************
// Funções externas do módulo não presentes em arquivos .h
extern void init_usart(void);
//extern void init_timer0(void);
extern void init_timer1(void);
extern void usart_putch(unsigned char mbyte);
extern void usart_putchSTR(char *mystr);
extern void bin_hex(byte binchar, byte *high_char, byte *low_char);

//******************************************************************************
// Funções do módulo

//******************************************************************************
// Função config pic - configura pinos de I/O e outros
//
// Entradas - nenhuma
// Saídas   - nenhuma
//******************************************************************************
void config_pic(void){
    
    PORTA = 0;                  // zera ports
    PORTB = 0;
        
    INTCON = 0x00;              // Todas as ints desligadas
    CMCON = 0x07;               // desliga comparadores 
    CCP1CONbits.CCP1M = 0b000;
    
    TRISA = 0xEF;               // Configura PORT A - entrada, exceto RA4
    TRISB = 0x82;               // Configura PORT B - saída, exceto RB1 e RB7
    
}

//*******************************************************************************
// Init vars - inicia variáveis
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*******************************************************************************
void init_vars(void){

	autoriza = 0;
	counter  = 0;
	prima = 0;
	envio = 0;
    
    mytimeout = 0;
    timeout_aut = 0;

    toTerm = 0;
    
#ifdef __DEBUG__
	debug = 0x30;
#endif
}

//******************************************************************************
// Função para ler as teclas
//
// Entradas - nenhuma
// Saídas   - valor da tecla ou 0 se sem tecla
//******************************************************************************
uint8_t getBTNs(void){
    
    uint8_t ret = 0;
    
    switch(TECLAS & 0x0F){      //lê port A e limpa MSB
        case 0x0D:              //se tecla S3 pressionada
            while((TECLAS & 0x0F)==0x0D);
            ret = 0x04;
            LED1 = 0;
            LED2 = 0;
            LED3 = 1;
            break;
        case 0x0B:              //se tecla S2 pressionada
            while((TECLAS & 0x0F)==0x0B)
            ret = 0x03;
            LED1 = 0;
            LED2 = 1;
            LED3 = 0;
            break;
        case 0x07:              //se tecla S1 pressionada
            while((TECLAS & 0x0F)==0x07)
            ret = 0x02;
            LED1 = 1;
            LED2 = 0;
            LED3 = 0;
            break;
    }
    
    return ret;
}

//******************************************************************************
// Função para enviar mensagem para um terminal
//
// Entradas - nr do terminal
// Saídas   - nenhuma
//******************************************************************************
void msgTerm(uint8_t toTerm){
    
    if(!prima){						// se esta recebendo, cancela envio!
		envio = 1;					// avisa que vai iniciar transmissão
        timeout_aut = 0;
        mytimeout = 0;
		TxRxbuf[0] = HEADER;				    //
#ifdef __DEBUG__
		uint8_t aux = NR_DISP;
		aux <<= 0x04;
		aux |= toTerm;
		bin_hex(aux, &mhigh, &mlow);
		TxRxbuf[1] = mhigh;
		TxRxbuf[2] = mlow;
        bin_hex((uint8_t)strlen("PE-PIC16F628A"), &mhigh, &mlow);
		TxRxbuf[3] = mhigh;	// quantidade de caracteres
        TxRxbuf[4] = mlow;
        TxRxbuf[5] = '\0';
		strcat(TxRxbuf,"PE-PIC16F628A");
        p[0] = CR;
        p[1] = LF;
        p[2] = '\0';
        strcat(TxRxbuf, p);
#else
        uint8_t aux = NR_DISP;
        aux <<= 0x04;
        aux |= toTerm;
        TxRxbuf[1] = aux;                       // número do dispositivo que envia
		TxRxbuf[2] = (char)strlen("PE-PIC16F628A");    // quantidade de caracteres
		TxRxbuf[3] = '\0';						// fecha para inserir dados
		strcat(TxRxbuf,"PE-PIC16F628A");		// insere MSG
#endif

		for(uint8_t i=0; i<strlen(TxRxbuf);i++)	//envia dados
			usart_putch(TxRxbuf[i]);
		
        envio = 0;								// avisa que está pronto para receber novamente!
        toTerm = 0;
	}

}

//******************************************************************************
// Função principal
//
// Entradas - nenhuma
// Saídas   - nenhuma
//******************************************************************************
void main(void) {
               
    char RxTerm[3];				// var para inserir dados recebidos do terminal
	char dados[TAMDATA];
#ifdef __DEBUG__
	uint8_t mhigh, mlow;
#endif
	 
    //**************************************************************************
    // Prepara PIC
    config_pic();               // configura pic
            
    //**************************************************************************
    // Prepara usart para uso
    init_usart();               //inicia USART
    
    //**************************************************************************
    // Prepara timer1 para uso
    init_timer1();
    
    SPI_init();                 // inicia SPI
    MAX7219_init();             // inicia matriz(es)
    MAX7219_clear();            // limpa matriz(es)
       
    MAX7219_displayText(" ", VERTICAL);
    __delay_ms(500);
    
    //MAX7219_scrollMMatrix("Bem-vindo");
    //MAX7219_scrollMMatrix("ao");
    //MAX7219_scrollMMatrix("Lab");
    //MAX7219_scrollMMatrix("do");
    //MAX7219_scrollMMatrix("Arne");

    LED1 = 0;                   // LEDs desligados
    LED2 = 0;
    LED3 = 0;
    
    //**************************************************************************
    // Flags de controle das interrupções
    INTCON |= 0x40;     // PEIE=1 - habilita int dos periféricos (timers, comparadores, usart, etc)
    INTCON |= 0x80;     // GIE=1 - habilita interrupções globais.
    
    MAX7219_clear();            // limpa matriz(es)
     
    while(TRUE){
               
        if(autoriza){							// tem dado??
			if(valida(&RxTerm[0], &dados[0])){  // testa protocolo - valida!
				envio = 1;
                timeout_aut = 0;
                mytimeout = 0;
                MAX7219_clear();            	// limpa matriz(es)
                MAX7219_scrollMMatrix(dados);   // mostra msg no painel

                TxRxbuf[0] = HEADER;			// devolve ACK
#ifdef __DEBUG__
				uint8_t aux = NR_DISP;
				aux <<= 0x04;
				aux |= RxTerm[0];
				bin_hex(aux, &mhigh, &mlow);
				TxRxbuf[1] = mhigh;
				TxRxbuf[2] = mlow;
				TxRxbuf[3] = 0x01+debug;		// quantidade de bytes
				TxRxbuf[4] = ACK;				// envia ACK
				TxRxbuf[5] = '\0';				// fecha para inserir dados
				TxRxbuf[6] = CR;
				TxRxbuf[7] = LF;
				TxRxbuf[8] = '\0';				// fecha string
#else
				uint8_t aux = NR_DISP;
                aux <<= 0x04;
                aux |= RxTerm[0];
                TxRxbuf[1] = aux;		// número do dispositivo que receberá
				TxRxbuf[2] = 0x01;				// quantidade de bytes
				TxRxbuf[3] = ACK;				// envia ACK
				TxRxbuf[4] = '\0';				// fecha string
#endif

				for(uint8_t i=0; i<strlen(TxRxbuf);i++)	//envia dados - ACK
					usart_putch(TxRxbuf[i]);
				envio = 0;								// avisa que está pronto para receber novamente!
			}
			autoriza = 0;						// autoriza novo buffer
		}    
	} 
    
    return;
}

